/**
* @file CAL Code to Syriac Unicode conversion
* @version 1.0.8
* @author Greg Borota
* @copyright (c) 2017 Greg Borota.
* @license MIT
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*/

// https://peshitta.github.io
// https://sedra.bethmardutho.org/about/fonts
// http://cal1.cn.huc.edu/searching/fullbrowser.html

import { Mapper, Writing } from 'aramaic-mapper';
import { allConsonants, diacritics, isBdwlPrefix, isVowel, vowels } from 'cal-code-util';
import { commonDiacritics, consonants, consonantsByName, easternCommonVowels, easternOnlyVowels, easternVowelsByName, westernVowels, westernVowelsByName } from 'syriac-code-util';

/** @module calSyriac */
/**
 * @private
 * CAL source writing
 * @const
 * @type { Writing }
 */
var calWriting = new Writing(allConsonants, vowels, diacritics);

var reversedPe = '\u0727';

/**
 * @private
 * Syriac destination writing with Eastern vowels
 * @const
 * @type { Writing }
 */
var easternSyriacWriting = new Writing(
  // + Palestinian and Hebrew Shin
  Object.freeze(
    consonants.concat(reversedPe, consonantsByName.semkath)
  ),
  Object.freeze(
    // duplicate the . under diacritic for i and u
    easternCommonVowels.concat(
      [easternVowelsByName.hbasaEsasa].concat(easternOnlyVowels)
    )
  ),
  commonDiacritics
);

/**
 * Aramaic Eastern Mapper
 * @const
 * @type { Mapper }
 */
var easternMapper = new Mapper(calWriting, easternSyriacWriting);

/**
 * @private
 * Syriac destination writing with Western vowels
 * @const
 * @type { Writing }
 */
var westernSyriacWriting = new Writing(
  // + Palestinian and Hebrew Shin
  Object.freeze(
    consonants.concat(reversedPe, consonantsByName.semkath)
  ),
  // CAL eastern short E and O mapped to western Rbasa and Esasa respectively
  Object.freeze(
    westernVowels.concat(
      westernVowelsByName.rbasa,
      westernVowelsByName.esasa
    )
  ),
  commonDiacritics
);

/**
 * @private
 * Return true Y|w+vowel is flipped a la Sedra
 * @param { string } word word to test
 * @param { number } index position in word to start checking from
 * @returns { boolean } true if Y|w+vowel is flipped
 */
var isYwFlipped = function (word, index) {
  if (index < 1) {
    return false; // can't start word with a vowel
  }
  if (isVowel(word.charAt(index - 1))) {
    return false; // can't follow vowel by vowel
  }
  if (word.charAt(index - 1) === ')') {
    return index === 1 || isBdwlPrefix(word, index - 2); // initial Alap is followed by vowel
  }
  return true;
};

/**
 * @private
 * Maps input character to Syriac char
 * @param { string } c input character
 * @param { Object.<string, string> } fromTo mapping dictionary
 * @returns { string } Syriac mapped char
 */
var map = function (c, fromTo) { return fromTo[c] || c; };

/**
 * @private
 * Customized mapping callback
 * @param { string } word input word
 * @param { number } i current index in the word
 * @param { Object.<string, string> } fromTo mapping dictionary
 * @returns { string } Syriac mapped char
 */
var westernCallback = function (word, i, fromTo) {
  var m = '';
  var c = word.charAt(i);
  switch (c) {
    case 'y':
      m =
        word.charAt(i + 1) === 'i'
          ? isYwFlipped(word, i)
            ? ("" + (westernVowelsByName.hbasa) + (consonantsByName.yod))
            : ("" + (consonantsByName.yod) + (westernVowelsByName.hbasa))
          : word.charAt(i + 1) === 'e'
            ? isYwFlipped(word, i)
              ? ("" + (westernVowelsByName.rbasa) + (consonantsByName.yod))
              : ("" + (consonantsByName.yod) + (westernVowelsByName.rbasa))
            : map(c, fromTo);
      break;
    case 'w':
      m =
        word.charAt(i + 1) === 'u'
          ? isYwFlipped(word, i)
            ? ("" + (westernVowelsByName.esasa) + (consonantsByName.waw))
            : ("" + (consonantsByName.waw) + (westernVowelsByName.esasa))
          : word.charAt(i + 1) === 'O'
            ? ("" + (westernVowelsByName.zqapha) + (consonantsByName.waw))
            : map(c, fromTo);
      break;
    default:
      m = map(c, fromTo);
      break;
  }
  return m;
};

/**
 * Aramaic Western Mapper
 * @const
 * @type { Mapper }
 */
var westernMapper = new Mapper(
  calWriting,
  westernSyriacWriting,
  westernCallback
);

/**
 * Convert from CAL to Eastern Syriac Unicode - only vowels differ from western
 * @static
 * @param {string} word input word in CAL code transliteration
 * @returns {string} the input word converted to Eastern Syriac Unicode
 */
var toEasternSyriac = function (word) { return easternMapper.map(word); };

/**
 * Convert from CAL to Western Syriac Unicode - only vowels differ from eastern
 * @static
 * @param {string} word input word in CAL code transliteration
 * @returns {string} the input word converted to Western Syriac Unicode
 */
var toWesternSyriac = function (word) { return westernMapper.map(word); };

export { easternMapper, westernMapper, toEasternSyriac, toWesternSyriac };
