/**
* @file CAL Code to Syriac Unicode conversion
* @version 1.0.8
* @author Greg Borota
* @copyright (c) 2017 Greg Borota.
* @license MIT
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*/

// https://peshitta.github.io
// https://sedra.bethmardutho.org/about/fonts
// http://cal1.cn.huc.edu/searching/fullbrowser.html

(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('aramaic-mapper'), require('cal-code-util'), require('syriac-code-util')) :
	typeof define === 'function' && define.amd ? define(['exports', 'aramaic-mapper', 'cal-code-util', 'syriac-code-util'], factory) :
	(factory((global.calSyriac = {}),global.aramaicMapper,global.calCodeUtil,global.syriacCodeUtil));
}(this, (function (exports,aramaicMapper,calCodeUtil,syriacCodeUtil) { 'use strict';

/** @module calSyriac */
/**
 * @private
 * CAL source writing
 * @const
 * @type { Writing }
 */
var calWriting = new aramaicMapper.Writing(calCodeUtil.allConsonants, calCodeUtil.vowels, calCodeUtil.diacritics);

var reversedPe = '\u0727';

/**
 * @private
 * Syriac destination writing with Eastern vowels
 * @const
 * @type { Writing }
 */
var easternSyriacWriting = new aramaicMapper.Writing(
  // + Palestinian and Hebrew Shin
  Object.freeze(
    syriacCodeUtil.consonants.concat(reversedPe, syriacCodeUtil.consonantsByName.semkath)
  ),
  Object.freeze(
    // duplicate the . under diacritic for i and u
    syriacCodeUtil.easternCommonVowels.concat(
      [syriacCodeUtil.easternVowelsByName.hbasaEsasa].concat(syriacCodeUtil.easternOnlyVowels)
    )
  ),
  syriacCodeUtil.commonDiacritics
);

/**
 * Aramaic Eastern Mapper
 * @const
 * @type { Mapper }
 */
var easternMapper = new aramaicMapper.Mapper(calWriting, easternSyriacWriting);

/**
 * @private
 * Syriac destination writing with Western vowels
 * @const
 * @type { Writing }
 */
var westernSyriacWriting = new aramaicMapper.Writing(
  // + Palestinian and Hebrew Shin
  Object.freeze(
    syriacCodeUtil.consonants.concat(reversedPe, syriacCodeUtil.consonantsByName.semkath)
  ),
  // CAL eastern short E and O mapped to western Rbasa and Esasa respectively
  Object.freeze(
    syriacCodeUtil.westernVowels.concat(
      syriacCodeUtil.westernVowelsByName.rbasa,
      syriacCodeUtil.westernVowelsByName.esasa
    )
  ),
  syriacCodeUtil.commonDiacritics
);

/**
 * @private
 * Return true Y|w+vowel is flipped a la Sedra
 * @param { string } word word to test
 * @param { number } index position in word to start checking from
 * @returns { boolean } true if Y|w+vowel is flipped
 */
var isYwFlipped = function (word, index) {
  if (index < 1) {
    return false; // can't start word with a vowel
  }
  if (calCodeUtil.isVowel(word.charAt(index - 1))) {
    return false; // can't follow vowel by vowel
  }
  if (word.charAt(index - 1) === ')') {
    return index === 1 || calCodeUtil.isBdwlPrefix(word, index - 2); // initial Alap is followed by vowel
  }
  return true;
};

/**
 * @private
 * Maps input character to Syriac char
 * @param { string } c input character
 * @param { Object.<string, string> } fromTo mapping dictionary
 * @returns { string } Syriac mapped char
 */
var map = function (c, fromTo) { return fromTo[c] || c; };

/**
 * @private
 * Customized mapping callback
 * @param { string } word input word
 * @param { number } i current index in the word
 * @param { Object.<string, string> } fromTo mapping dictionary
 * @returns { string } Syriac mapped char
 */
var westernCallback = function (word, i, fromTo) {
  var m = '';
  var c = word.charAt(i);
  switch (c) {
    case 'y':
      m =
        word.charAt(i + 1) === 'i'
          ? isYwFlipped(word, i)
            ? ("" + (syriacCodeUtil.westernVowelsByName.hbasa) + (syriacCodeUtil.consonantsByName.yod))
            : ("" + (syriacCodeUtil.consonantsByName.yod) + (syriacCodeUtil.westernVowelsByName.hbasa))
          : word.charAt(i + 1) === 'e'
            ? isYwFlipped(word, i)
              ? ("" + (syriacCodeUtil.westernVowelsByName.rbasa) + (syriacCodeUtil.consonantsByName.yod))
              : ("" + (syriacCodeUtil.consonantsByName.yod) + (syriacCodeUtil.westernVowelsByName.rbasa))
            : map(c, fromTo);
      break;
    case 'w':
      m =
        word.charAt(i + 1) === 'u'
          ? isYwFlipped(word, i)
            ? ("" + (syriacCodeUtil.westernVowelsByName.esasa) + (syriacCodeUtil.consonantsByName.waw))
            : ("" + (syriacCodeUtil.consonantsByName.waw) + (syriacCodeUtil.westernVowelsByName.esasa))
          : word.charAt(i + 1) === 'O'
            ? ("" + (syriacCodeUtil.westernVowelsByName.zqapha) + (syriacCodeUtil.consonantsByName.waw))
            : map(c, fromTo);
      break;
    default:
      m = map(c, fromTo);
      break;
  }
  return m;
};

/**
 * Aramaic Western Mapper
 * @const
 * @type { Mapper }
 */
var westernMapper = new aramaicMapper.Mapper(
  calWriting,
  westernSyriacWriting,
  westernCallback
);

/**
 * Convert from CAL to Eastern Syriac Unicode - only vowels differ from western
 * @static
 * @param {string} word input word in CAL code transliteration
 * @returns {string} the input word converted to Eastern Syriac Unicode
 */
var toEasternSyriac = function (word) { return easternMapper.map(word); };

/**
 * Convert from CAL to Western Syriac Unicode - only vowels differ from eastern
 * @static
 * @param {string} word input word in CAL code transliteration
 * @returns {string} the input word converted to Western Syriac Unicode
 */
var toWesternSyriac = function (word) { return westernMapper.map(word); };

exports.easternMapper = easternMapper;
exports.westernMapper = westernMapper;
exports.toEasternSyriac = toEasternSyriac;
exports.toWesternSyriac = toWesternSyriac;

Object.defineProperty(exports, '__esModule', { value: true });

})));
